var rotate = 0;
var isMuted = false;
var isPlaying = true;
var hasVolume = false;

var deviceid;
var signalserver;

const querystring = window.location.search;
const streamid = querystring.slice(10); // skip "?streamid="
console.log(streamid);

// Get device id
var request = new XMLHttpRequest();

request.open('GET', 'getdeviceid', false);
request.onload = function () {
    if (request.status >= 200 && request.status < 400) {
        var data = JSON.parse(this.response);
        deviceid = data.deviceid;
        console.log("deviceid : " + deviceid);
    } else {
        console.log('get device id error!!!');
    }
}

request.send();

// Get signal server url
var request1 = new XMLHttpRequest();

request1.open('GET', 'getsignalserver', false);
request1.onload = function () {
    if (request1.status >= 200 && request1.status < 400) {
        var data = JSON.parse(this.response);
        signalserver = data.signalserver;
        console.log("signal server : " + signalserver);
    } else {
        console.log('get signal server error!!!');
    }
}

request1.send();

window.onunload = function() {
    onStop()
}

window.onbeforeunload = function() {
    onStop()
}

var pc;

const log = msg => {
    console.log(msg)
}

function setupPeerConnection(iceServers1)
{
    pc = new RTCPeerConnection({
        iceServers: iceServers1
        });
    
    console.log("protocol = " + document.location.protocol);
    
    if (document.location.protocol == "https:") {
        // Request mic access
        navigator.mediaDevices.getUserMedia({ video: false, audio: true })
            .then(stream => {
                stream.getTracks().forEach(track => pc.addTrack(track, stream));
        }).catch(log);
    }

    pc.onicecandidate = event => {
        if (event.candidate === null) {
            console.log("local sdp : " + pc.localDescription.sdp);
            
            let json = {
                "jsonrpc": "2.0", "result": {"answer": pc.localDescription.sdp}, "id": remotemsgid
            }
            ws.send(JSON.stringify(json));
        }
    }
    
    pc.ontrack = function (event) {
        console.log("track.kind:"+event.track.kind);
        
        if (event.track.kind == 'video') {
            var el = document.getElementById('videoStream');
            
            if (el.srcObject == null)
            {
                var newStream = new MediaStream();
                newStream.addTrack(event.track);
                el.srcObject = newStream;
                el.autoplay = true;
                el.controls = false;
                el.muted = true;
            }
            
            document.getElementById('videoStream').style.display = 'block';
            
        } else if (event.track.kind == 'audio') {
            var el = document.getElementById('audioStream');
            
            if (el.srcObject == null)
            {
                var newStream = new MediaStream();
                newStream.addTrack(event.track);
                el.srcObject = newStream;
                el.controls = false;
                el.muted = false;
            }
        }
    }
    
    pc.onaddstream = function (e) {
        console.log("onaddstream");
    }
    
    pc.oniceconnectionstatechange = e => {
        log(pc.iceConnectionState)
        document.getElementById('status').innerHTML = pc.iceConnectionState;
        
        if (pc.iceConnectionState == 'connected') {
            // default to muted
            if (!isMuted)
                onMuted();
        }
    }    
}

function startConnection(iceServers)
{
    setupPeerConnection(iceServers);
}

const STATE_REGISTERING=1;
const STATE_CONNECTING=2;
const STATE_CONNECTED=3;
const STATE_CLOSED=4;

var ws = new WebSocket(signalserver);
var uid;
var state = 0;
var localmsgid = 1;
var remotemsgid;
var session;

ws.onopen = function(evt) {
    let data = {
        "jsonrpc": "2.0", "method": "register", "params":  {"authorization": ""}, "id": String(localmsgid)
    }
    ws.send(JSON.stringify(data));
    
    state = STATE_REGISTERING;
    localmsgid++;
    console.log("data sending : " + JSON.stringify(data));
}

ws.onmessage = function(evt) { 
    var msg = evt.data;
    console.log("data received:" + msg);
    
    var msg1 = msg.replace(/\n/g,"\\n").replace(/\r/g,"\\r");
                                   
    var data = JSON.parse(msg1);
    
    if (state == STATE_REGISTERING)
    {
        uid = data.result.id;
        console.log("uid : " + uid);
        
        let senddata = {
            "jsonrpc": "2.0", "method": "connect", "params":  {"authorization": "", "deviceid": deviceid, "streamid": streamid}, "id": String(localmsgid)
        }
        ws.send(JSON.stringify(senddata));
    
        state = STATE_CONNECTING;
        localmsgid++;
        console.log("data sending : " + JSON.stringify(senddata));
    }
    else if (state == STATE_CONNECTING)
    {
        if ("result" in data)
        {
            session = data.result.session;
            console.log("session : " + session);
            
            startConnection(data.result.iceServers);
            
            state = STATE_CONNECTED;
        }
        else if ("method" in data)
        {
            if (data.method == "trickle")
            {
            }    
        }    
    }
    else if (state == STATE_CONNECTED)
    {
        if ("method" in data)
        {
            if (data.method == "invite")
            {
                remotemsgid = data.id;
                
                var sdp = data.params.offer;
                
                let offer = {type: 'offer', sdp: sdp};
                
                pc.setRemoteDescription(offer);
                pc.createAnswer().then(d => pc.setLocalDescription(d)).catch(log);
            }
            else if (data.method == "trickle")
            {
            }
        }
    }
}

ws.onclose = function(evt)
{ 
    console.log("web socket closed..."); 
}

function onVolume() {
    var audioStream = document.getElementById('audioStream');
    hasVolume = !hasVolume;
    
    if (hasVolume) {
        audioStream.play();
    } else {
        audioStream.pause();
    }
}

function onRotate() {
    rotate = rotate + 180;
    
    if (rotate == 360) {
        rotate = 0;
    }
    
    document.getElementById('videoStream').style.transform = 'rotate(' + rotate + 'deg)';
}

function onMuted() {
    isMuted = !isMuted;
    
    if (document.location.protocol == "http:")
    {
        return;
    }
    
    if (isMuted) {
        
        pc.getSenders().forEach(function (sender) {
            // stop audio track
            if (sender.track != null && sender.track.kind == 'audio') {
                sender.track.stop();
            }
        })
        
        document.getElementById('mute').innerHTML = "Microphone";
    } else {
    
        // access mic again
        navigator.mediaDevices.getUserMedia({ video: false, audio: true })
            .then(stream => {
                stream.getTracks().forEach(track => {
                pc.getSenders().forEach(function (sender) {
                    if (sender.track != null && sender.track.kind == 'audio') {
                        sender.replaceTrack(track);
                    }
                })
            });
        }).catch(log);
        
        document.getElementById('mute').innerHTML = "Mute";
    }
}

function onStop() {
    isPlaying = !isPlaying;
    
    if (isPlaying) {
        
    } else if (state != STATE_CLOSED) {
    
        let senddata = {
            "jsonrpc": "2.0", "method": "close", "params":  {"session": session}, "id": String(localmsgid)
        }
        ws.send(JSON.stringify(senddata));
    
        state = STATE_CLOSED;
        localmsgid++;
        console.log("data sending : " + JSON.stringify(senddata));
    }
}

